@@ -34,6 +34,9 @@ from hsfs import (
     util,
 )
 from hsfs.constructor.query import Query
+from hsfs.core import (
+    data_source as ds,
+)
 from hsfs.core import (
     feature_group_api,
     feature_group_engine,
@@ -532,6 +535,12 @@ class FeatureStore:
             storage_connector.StorageConnector, Dict[str, Any]
         ] = None,
         path: Optional[str] = None,
+        data_source: Optional[
+            Union[
+                ds.DataSource,
+                Dict[str, Any],
+            ]
+        ] = None,
     ) -> feature_group.FeatureGroup:
         """Create a feature group metadata object.
 
@@ -643,10 +652,13 @@ class FeatureStore:
                 with the data source.
             path: The location within the scope of the storage connector, from where to read
                 the data for the external feature group
+            data_source: The data source specifying the location of the data. Overrides the path and query arguments when specified.
 
         # Returns
             `FeatureGroup`. The feature group metadata object.
         """
+        if not data_source:
+            data_source = ds.DataSource(path=path)
         feature_group_object = feature_group.FeatureGroup(
             name=name,
             version=version,
@@ -672,7 +684,7 @@ class FeatureStore:
             online_config=online_config,
             offline_backfill_every_hr=offline_backfill_every_hr,
             storage_connector=storage_connector,
-            path=path,
+            data_source=data_source,
         )
         feature_group_object.feature_store = self
         return feature_group_object
@@ -712,6 +724,12 @@ class FeatureStore:
             storage_connector.StorageConnector, Dict[str, Any]
         ] = None,
         path: Optional[str] = None,
+        data_source: Optional[
+            Union[
+                ds.DataSource,
+                Dict[str, Any],
+            ]
+        ] = None,
     ) -> Union[
         feature_group.FeatureGroup,
         feature_group.ExternalFeatureGroup,
@@ -813,12 +831,15 @@ class FeatureStore:
                 with the data source.
             path: The location within the scope of the storage connector, from where to read
                 the data for the external feature group
+            data_source: The data source specifying the location of the data. Overrides the path and query arguments when specified.
 
         # Returns
             `FeatureGroup`. The feature group metadata object.
         """
         feature_group_object = self._feature_group_api.get(self.id, name, version)
         if not feature_group_object:
+            if not data_source:
+                data_source = ds.DataSource(path=path)
             feature_group_object = feature_group.FeatureGroup(
                 name=name,
                 version=version,
@@ -844,7 +865,7 @@ class FeatureStore:
                 online_config=online_config,
                 offline_backfill_every_hr=offline_backfill_every_hr,
                 storage_connector=storage_connector,
-                path=path,
+                data_source=data_source,
             )
         feature_group_object.feature_store = self
         return feature_group_object
@@ -873,6 +894,12 @@ class FeatureStore:
         ] = None,
         topic_name: Optional[str] = None,
         notification_topic_name: Optional[str] = None,
+        data_source: Optional[
+            Union[
+                ds.DataSource,
+                Dict[str, Any],
+            ]
+        ] = None,
     ) -> feature_group.ExternalFeatureGroup:
         """Create an external feature group metadata object.
 
@@ -935,6 +962,7 @@ class FeatureStore:
             expectation_suite: Optionally, attach an expectation suite to the feature
                 group which dataframes should be validated against upon insertion.
                 Defaults to `None`.
+            data_source: The data source specifying the location of the data. Overrides the path and query arguments when specified.
 
 
 
@@ -943,11 +971,11 @@ class FeatureStore:
         # Returns
             `ExternalFeatureGroup`. The external feature group metadata object.
         """
+        if not data_source:
+            data_source = ds.DataSource(query=query, path=path)
         feature_group_object = feature_group.ExternalFeatureGroup(
             name=name,
-            query=query,
             data_format=data_format,
-            path=path,
             options=options or {},
             storage_connector=storage_connector,
             version=version,
@@ -962,6 +990,7 @@ class FeatureStore:
             expectation_suite=expectation_suite,
             topic_name=topic_name,
             notification_topic_name=notification_topic_name,
+            data_source=data_source,
         )
         feature_group_object.feature_store = self
         return feature_group_object
@@ -998,6 +1027,12 @@ class FeatureStore:
                 Dict[str, Any],
             ]
         ] = None,
+        data_source: Optional[
+            Union[
+                ds.DataSource,
+                Dict[str, Any],
+            ]
+        ] = None,
     ) -> feature_group.ExternalFeatureGroup:
         """Create an external feature group metadata object.
 
@@ -1102,15 +1137,16 @@ class FeatureStore:
             notification_topic_name: Optionally, define the name of the topic used for sending notifications when entries
                 are inserted or updated on the online feature store. If left undefined no notifications are sent.
             online_config: Optionally, define configuration which is used to configure online table.
+            data_source: The data source specifying the location of the data. Overrides the path and query arguments when specified.
 
         # Returns
             `ExternalFeatureGroup`. The external feature group metadata object.
         """
+        if not data_source:
+            data_source = ds.DataSource(query=query, path=path)
         feature_group_object = feature_group.ExternalFeatureGroup(
             name=name,
-            query=query,
             data_format=data_format,
-            path=path,
             options=options or {},
             storage_connector=storage_connector,
             version=version,
@@ -1128,6 +1164,7 @@ class FeatureStore:
             topic_name=topic_name,
             notification_topic_name=notification_topic_name,
             online_config=online_config,
+            data_source=data_source,
         )
         feature_group_object.feature_store = self
         return feature_group_object
