@@ -618,10 +618,12 @@ class TestKafkaConnector:
 
     def test_spark_options(self, mocker, backend_fixtures):
         # Arrange
-        mocker.patch("hsfs.engine.get_instance")
+        mock_engine_get_instance = mocker.patch("hsfs.engine.get_instance")
         mock_client_get_instance = mocker.patch("hopsworks_common.client.get_instance")
         json = backend_fixtures["storage_connector"]["get_kafka_internal"]["response"]
 
+        mock_engine_get_instance.return_value.get_spark_version.return_value = "3.1.0"
+
         mock_client_get_instance.return_value._get_jks_trust_store_path.return_value = (
             "result_from_get_jks_trust_store_path"
         )
@@ -653,6 +655,7 @@ class TestKafkaConnector:
         mock_engine_get_instance = mocker.patch("hsfs.engine.get_instance")
         json = backend_fixtures["storage_connector"]["get_kafka_external"]["response"]
 
+        mock_engine_get_instance.return_value.get_spark_version.return_value = "3.1.0"
         mock_engine_get_instance.return_value.add_file.return_value = (
             "result_from_add_file"
         )
@@ -675,6 +678,53 @@ class TestKafkaConnector:
             "kafka.ssl.key.password": "test_ssl_key_password",
         }
 
+    def test_spark_options_spark_35(self, mocker, backend_fixtures):
+        # Arrange
+        mock_engine_get_instance = mocker.patch("hsfs.engine.get_instance")
+        mock_client_get_instance = mocker.patch("hopsworks_common.client.get_instance")
+        json = backend_fixtures["storage_connector"]["get_kafka_internal"]["response"]
+
+        mock_engine_get_instance.return_value.get_spark_version.return_value = "3.5.0"
+
+        mock_client_get_instance.return_value._get_jks_trust_store_path.return_value = (
+            "result_from_get_jks_trust_store_path"
+        )
+        mock_client_get_instance.return_value._get_jks_key_store_path.return_value = (
+            "result_from_get_jks_key_store_path"
+        )
+        mock_client_get_instance.return_value._cert_key = "result_from_cert_key"
+        mock_client_get_instance.return_value._write_pem.return_value = (
+            None,
+            None,
+            None,
+        )
+
+        sc = storage_connector.StorageConnector.from_response_json(json)
+
+        # Mock the read pem method in the storage connector itself
+        sc._read_pem = mocker.Mock()
+        sc._read_pem.side_effect = [
+            "test_ssl_ca",
+            "test_ssl_certificate",
+            "test_ssl_key",
+        ]
+
+        # Act
+        config = sc.spark_options()
+
+        # Assert
+        assert config == {
+            "kafka.test_option_name": "test_option_value",
+            "kafka.bootstrap.servers": "test_bootstrap_servers",
+            "kafka.security.protocol": "test_security_protocol",
+            "kafka.ssl.endpoint.identification.algorithm": "test_ssl_endpoint_identification_algorithm",
+            "kafka.ssl.truststore.type": "PEM",
+            "kafka.ssl.keystore.type": "PEM",
+            "kafka.ssl.truststore.certificates": "test_ssl_ca",
+            "kafka.ssl.keystore.certificate.chain": "test_ssl_certificate",
+            "kafka.ssl.keystore.key": "test_ssl_key",
+        }
+
     def test_confluent_options(self, mocker, backend_fixtures):
         # Arrange
         mock_engine_get_instance = mocker.patch("hsfs.engine.get_instance")
